# -----------------------------------------------------------
# This script intersects a city shapefile with buffered
# MRT stops
#
# Vincent Verbavatz - 2020
# -----------------------------------------------------------

import os, sys
import matplotlib.pyplot as plt
from osgeo import gdal, ogr, osr
import pandas as pd
from geotools import *

def remove_layer(layer):
    os.remove(layer + '.shp')
    os.remove(layer + '.shx')
    os.remove(layer + '.dbf')
    os.remove(layer + '.prj')

def intersect_network(cityname, buffer_dists):
    """
    Calculates share of population living at different distances from MRT in the
    corresponding city
    """

    folder = cityname
    cityname = os.path.basename(cityname)
    columns = ('City' , *[str(x) + 'm' for x in buffer_dists])
    pc_values = pd.DataFrame(columns=columns)

    drv = ogr.GetDriverByName("ESRI Shapefile")
    csv = ogr.GetDriverByName("CSV")

    #Defines mollweide projection from well known text
    mollweide_wkt = 'PROJCS["World_Mollweide",GEOGCS["WGS 84",DATUM["WGS_1984", \
                    SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]], \
                    AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0],UNIT["Degree",0.0174532925199433]], \
                    PROJECTION["Mollweide"],PARAMETER["central_meridian",0],PARAMETER["false_easting",0], \
                    PARAMETER["false_northing",0],UNIT["metre",1,AUTHORITY["EPSG","9001"]], \
                    AXIS["Easting",EAST],AXIS["Northing",NORTH]]'
    mollweide = osr.SpatialReference()
    mollweide.ImportFromWkt(mollweide_wkt)

    nameField = ogr.FieldDefn('Name', ogr.OFTString)
    typeField = ogr.FieldDefn('Type', ogr.OFTInteger)
    popField = ogr.FieldDefn('Population', ogr.OFTReal)
    densityField = ogr.FieldDefn('Density', ogr.OFTReal)

    #Imports MRT stop shapefile
    mrt_ds = drv.Open(folder + "/stops.shp", update = 1)
    mrt_layer = mrt_ds.GetLayer()

    #Imports city with population-grid
    pop_ds = drv.Open(folder + "/" + cityname + ".shp", update = 1)
    pop_layer = pop_ds.GetLayer()

    #Calculates total city population
    total_pop = sum_pop(pop_layer)

    values = [cityname]
    for buffer_dist in buffer_dists:
        #checks if there is a MRT system in the city
        if mrt_layer.GetFeatureCount() > 0:
            #prepares shapefile of buffered stops
            buffer_ds = drv.CreateDataSource(folder + "/buffer_mrt_" + str(buffer_dist) + ".shp")
            buffer_layer = buffer_ds.CreateLayer('buffer_mrt_' + str(buffer_dist),
            geom_type=ogr.wkbPolygon, srs = mollweide)

            #prepare interesection of buffered stops with population grid
            dst_ds = drv.CreateDataSource(folder + "/" + str(buffer_dist) + "_" + cityname + ".shp")
            intersection_layer = dst_ds.CreateLayer('intersection_' + str(buffer_dist),
            geom_type=ogr.wkbPolygon, srs = mollweide)
            intersection_layer.CreateField(popField)
            intersection_layer.CreateField(densityField)

            #Adds density field to the population-grid
            density(pop_layer)

            #buffers the MRT stops
            createBuffer(mrt_layer, buffer_layer, buffer_dist)

            #intersects pouplation-grid and buffers
            pop_layer.Intersection(buffer_layer, intersection_layer)

            #recalculates population
            inverse_density(intersection_layer)

            #sums up population in the total MRT-covered area
            values.append(str(round(sum_pop(intersection_layer) / total_pop * 100, 2)))

            buffer_ds.Destroy()
            dst_ds.Destroy()
            remove_layer(folder + "/buffer_mrt_" + str(buffer_dist))

        else:
            values.append('0')

    #Adds new row to output file
    new_row = dict(zip(columns, values))
    pc_values = pc_values.append(new_row, ignore_index=True)

    mrt_ds.Destroy()
    pop_ds.Destroy()

    pc_values.to_csv(folder + '/mrt_dist_ '+ cityname +'.csv', index=False)
    print(pc_values)
