# -----------------------------------------------------------
# This script converts a GTFS zip into a shapefile with elements
# MRT stops.
#
# Vincent Verbavatz - 2020
# -----------------------------------------------------------

import os, sys
from zipfile import ZipFile, BadZipfile
from osgeo import gdal, ogr, osr
import pandas as pd
from geotools import createBuffer

def convert_zip(zip_path, city_path):
    """
    Associates stops with route_type and saves them to a CSV file.
    """
    try:
        with ZipFile(zip_path, 'r') as zipObj:
            zipObj.extractall(city_path + '/')

        #reads routes, trips, stop_times and stops files to prepare merging
        routes = pd.read_csv(city_path + '/routes.txt')
        trips = pd.read_csv(city_path + '/trips.txt')
        stop_times = pd.read_csv(city_path + '/stop_times.txt', dtype={"stop_id":str})
        stops = pd.read_csv(city_path + '/stops.txt', dtype={"stop_id":str})

        #associates trips with their corresponding route type
        typed_trips = trips.merge(routes[['route_id', 'route_type']])[['route_id',
         'trip_id', 'route_type']]

        #associates stop times with their corresponding route type
        typed_stop_times = stop_times.merge(typed_trips[['trip_id', 'route_type']],
        how='left')[['trip_id', 'stop_id', 'route_type']]
        typed_stop_times = typed_stop_times.drop_duplicates()

        #associates stops with their corresponding route type
        stops["stop_id"] = stops["stop_id"].str.lstrip("0 ")
        typed_stop_times["stop_id"] = typed_stop_times["stop_id"].str.lstrip("0 ")
        typed_stops = stops.merge(typed_stop_times[['stop_id', 'route_type']], how='inner')
        typed_stops = typed_stops.drop_duplicates()

        typed_stops.to_csv(city_path + '/stops.csv', index=False)

        for file in os.listdir(city_path):
            if file.endswith('.txt'):
                os.remove(city_path + '/' + file)

    except BadZipfile:
        print("Bad Zip")

def process_gtfs(zip_path, city_path):
    """
    Convers a GTFS zipfile into a shapefile with points the MRT (no bus nor ferry) stops.
    """
    convert_zip(zip_path, city_path)

    #Defines mollweide projection from well known text
    mollweide_wkt = 'PROJCS["World_Mollweide",GEOGCS["WGS 84",DATUM["WGS_1984", \
                    SPHEROID["WGS 84",6378137,298.257223563,AUTHORITY["EPSG","7030"]], \
                    AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0],UNIT["Degree",0.0174532925199433]], \
                    PROJECTION["Mollweide"],PARAMETER["central_meridian",0],PARAMETER["false_easting",0], \
                    PARAMETER["false_northing",0],UNIT["metre",1,AUTHORITY["EPSG","9001"]], \
                    AXIS["Easting",EAST],AXIS["Northing",NORTH]]'
    mollweide = osr.SpatialReference()
    mollweide.ImportFromWkt(mollweide_wkt)

    drv = ogr.GetDriverByName("ESRI Shapefile")
    csv = ogr.GetDriverByName("CSV")

    #Use World Geodetic System 1984 to import stop coordinates
    inSpatialRef = osr.SpatialReference()
    inSpatialRef.ImportFromEPSG(4326)

    #Defines Coordinate transformation
    coordTrans = osr.CoordinateTransformation(inSpatialRef, mollweide)

    nameField = ogr.FieldDefn('Name', ogr.OFTString)
    typeField = ogr.FieldDefn('Type', ogr.OFTInteger)

    cityname = city_path

    mrt_csv = csv.Open(city_path + "/stops.csv", update = 0)
    csv_layer = mrt_csv.GetLayer()

    shp_ds = drv.CreateDataSource(city_path + "/stops.shp")

    #Prepares MRT output layer
    mrt_layer = shp_ds.CreateLayer(cityname, geom_type=ogr.wkbPoint, srs = mollweide)
    mrt_layer.CreateField(nameField)
    mrt_layer.CreateField(typeField)
    featureDefn = mrt_layer.GetLayerDefn()

    #adds MRT points to the output file
    for feature in csv_layer:
        point = ogr.Geometry(ogr.wkbPoint)
        point.AddPoint(float(feature.GetField("stop_lat")), float(feature.GetField("stop_lon")))

        #converts to Mollweide
        point.Transform(coordTrans)

        outFeature = ogr.Feature(featureDefn)
        outFeature.SetGeometry(point)
        outFeature.SetField('Name', feature.GetField("stop_name"))
        outFeature.SetField('Type', feature.GetField("route_type"))

        #excludes buses and ferries
        if (feature.GetField("route_type") != "") and (feature.GetField("route_type")[:1] != "3") and (feature.GetField("route_type")[:1] != "7"):
            mrt_layer.CreateFeature(outFeature)

        outFeature = None

    print("{}: {} MRT stops".format(cityname, mrt_layer.GetFeatureCount()))
    shp_ds.Destroy()
